import argparse
import glob
import os
import re
from typing import Any

from rich.console import Console

from anyerplint.tnex import parse, translate
from anyerplint.util import open_text_file

console = Console()


def emit(s: str) -> None:
    console.print(s)


def print_recursive_lists(tree: Any, indent: int) -> None:
    first = True
    for el in tree:
        if isinstance(el, list):
            print_recursive_lists(el, indent + 1)
        else:
            t = "[deep_sky_blue1]" + el + "[/]" if first else el
            emit("  " * indent + t)
            first = False


def handle_parse(args: argparse.Namespace) -> None:
    for pat in args.filename:
        if os.path.isdir(pat):
            pat = pat + "/**/*.xml"
        fnames = glob.glob(pat, recursive=True) if "*" in pat else [pat]
        for f in fnames:
            console.rule(f)
            lines = open_text_file(f).readlines()
            for i, line_in in enumerate(lines):
                line = line_in.strip()
                expressions = re.findall("{(.*?)}", line)
                if not expressions:
                    continue
                for exp in expressions:
                    try:
                        parsed = parse(exp)
                        translated = translate(parsed[0])
                    except ValueError:
                        emit("[red]Failed to parse[/]: " + exp)
                        continue

                    if translated != exp:
                        emit(f"L[yellow]{i+1}[/]\t[darkgray]{exp}[/])")
                        emit("=>\t" + translated)
                        print_recursive_lists(parsed[0], 1)


def init_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    """Declare arguments you need here."""
    parser.add_argument("filename", nargs="+", help="Files to parse")
    parser.set_defaults(func=handle_parse)
    return parser



from anyerplint.tnex import parse, translate_str


def test_parser() -> None:
    assert parse("foo") == ["foo"]
    assert parse("foo(1;1)") == [["foo", "1", "1"]]
    assert parse('"some string"') == ['"some string"']
    assert parse('"foo";0') == ['"foo"', "0"]
    assert parse("foo(1);2") == [["foo", "1"], "2"]
    assert parse('foo("some string";"other string";0)') == [
        ["foo", '"some string"', '"other string"', "0"]
    ]
    assert parse('foo("some string";"other string");1') == [
        ["foo", '"some string"', '"other string"'],
        "1",
    ]
    # nothing special with ,
    assert parse("a,b(c,d;e,f;1.0)") == [["a,b", "c,d", "e,f", "1.0"]]

    assert parse(r'" string with \" char"') == ['" string with \\" char"']
    assert parse(r'"some string with ) and ( and \" characters"') == [
        '"some string with ) and ( and \\" characters"'
    ]

    assert parse(r'foo,"long / () accessor"') == ['foo,"long / () accessor"']
    assert parse(r'foo(F,NOW(),"yyyy")') == [["foo", [',"yyyy"', ["F,NOW"]]]]
    assert parse(r'F,NOW(),"yyyy"') == [[',"yyyy"', ["F,NOW"]]]

    complex = parse(
        r'foo(f1;bar(b1;b2;"b3";"wide string");f2;baz(1;"some string with ) and ( and \" characters")));top1'
    )
    assert complex == [
        [
            "foo",
            "f1",
            ["bar", "b1", "b2", '"b3"', '"wide string"'],
            "f2",
            ["baz", "1", '"some string with ) and ( and \\" characters"'],
        ]
    ]


def test_translate_syntax() -> None:
    assert (
        translate_str(
            'F,EVAL(obj.A,AssetCode;=;"";F,EVAL(obj.A,SUM;"&gt;";"0";40;50);F,EVAL(obj.A,SUM;"&gt;";"0";70;75))'
        )
        == '(obj.A,AssetCode = "" ? (obj.A,SUM > "0" ? 40 : 50) : (obj.A,SUM > "0" ? 70 : 75))'
    )

    assert (
        translate_str(
            "F,EVAL(F,EXISTS(v;config);=;1;F,REPLACE(F,LOWER(v,config);rootconfig;systemvars);&quot;&quot;)"
        )
        == '((config in v) = 1 ? v,config.lower().replace(rootconfig -> systemvars) : "")'
    )
    # parsed = parse(p2)
    # print(translate(parsed[0]))
